/************************************************************
 **
 ** slist.h : generic singly linked list declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef slist_h__
#define slist_h__

#include "ptype.h"

struct slist_el_;
typedef void (*slist_el_free_fn_t)(struct slist_el_ *el);

typedef struct slist_el_ {
  struct slist_el_  *link;
  struct slist_     *owner;
  refct_t            ref_ct;
  slist_el_free_fn_t free_fn;
} slist_el_t;

typedef struct slist_ {
  slist_el_t *head;
  slist_el_t *tail;
  unsigned    ct;
} slist_t;

void slist_init(slist_t *dst);
void slist_reset(slist_t *lst);
void slist_append(slist_t *lst, slist_el_t *el);
void slist_insert(slist_t *lst, slist_el_t *after, slist_el_t *el);
void slist_reset(slist_t *lst);
void slist_insert_list(slist_t *dst, slist_el_t *el, slist_t *src);
void slist_append_list(slist_t *dst, slist_t *src);

slist_el_t *slist_head(slist_t *lst);
slist_el_t *slist_tail(slist_t *lst);

typedef result_t(*slist_el_find_fn_t)(void *arg, slist_el_t *el, 
    const void *data);
slist_el_t *slist_find(slist_t *lst, slist_el_find_fn_t fn, void *arg, 
  const void *data);

unsigned slist_ct_get(const slist_t *lst);

void  slist_el_init(slist_el_t *lst, slist_el_free_fn_t fn);
void  slist_el_lock(slist_el_t *el);
void  slist_el_release(slist_el_t *el);
void *slist_el_data(slist_el_t *el);
slist_el_t *slist_el_next(const slist_el_t *el);
void  slist_el_remove(slist_el_t *el);
refct_t slist_el_refct_get(const slist_el_t *el);

#endif /* slist_h__ */

