/************************************************************
 **
 ** value.h : value declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef value_h__
#define value_h__

#include "variable.h" 
/* notes:
 *   var -- as expected, the base
 *   ofs -- for array variables, the offset from the base
 *             operator '_h' will also change the base
 *   sz  -- resulting size
 *             operators '_h' and '_l' reduce the size by 1/2\
 */
typedef struct value_ value_t;

/* dummy is used when adding or subtracting two variables in different
   banks. since this isn't easy, we just make one of them a pseudo-array */
#define VALUE_SUBSCRIPT_DUMMY ((variable_t *) -1)

result_t value_alloc(value_t **dst, variable_t *var, variable_t *subscript,
    variable_base_t baseofs, variable_sz_t sz);
void value_lock(value_t *val);
boolean_t value_same(const value_t *val1, const value_t *val2);

void     value_release(value_t *val);
result_t value_clone(value_t **dst, value_t *src);

variable_ct_t value_ct_get(const value_t *val);
void          value_ct_set(value_t *val, variable_ct_t ct);

variable_ct_t value_ct_ofs_get(const value_t *val);
void          value_ct_ofs_set(value_t *val, variable_ct_t ofs);

variable_t   *value_variable_get(const value_t *val);
void          value_variable_set(value_t *val, variable_t *var);
void          value_sz_set(value_t *val, variable_sz_t sz);
variable_sz_t value_sz_get(const value_t *val);
boolean_t     value_flag_test(const value_t *val, flag_t flags);
void          value_flag_set(value_t *val, flag_t flags);
void          value_flag_clr(value_t *val, flag_t flags);
flag_t        value_flags_get_all(value_t *val);
void          value_flags_set_all(value_t *val, flag_t flag);
const char   *value_name_get(const value_t *val);

variable_t   *value_subscript_get(value_t *val);
void          value_subscript_set(value_t *val, variable_t *subscript);

variable_base_t value_baseofs_get(const value_t *val);
void            value_baseofs_set(value_t *val, variable_base_t base);

variable_const_t value_const_get(const value_t *val);
void             value_const_set(value_t *val, variable_const_t c);

ctr_t          value_expr_ct_get(const value_t *val);
void           value_expr_ct_bump(value_t *var, ctr_bump_t dir);

ctr_t          value_assign_ct_get(const value_t *var);
void           value_assign_ct_bump(value_t *var, ctr_bump_t dir);

ctr_t          value_use_ct_get(const value_t *var);
void           value_use_ct_bump(value_t *var, ctr_bump_t dir);

void value_dump(const value_t *val, FILE *dst);
variable_base_t value_base_get(const value_t *var);

boolean_t value_is_const(const value_t *val);
boolean_t value_is_lookup(const value_t *val);
boolean_t value_is_array(const value_t *val);

#ifdef MSDOS
#include "val_decl.h"
#endif

#endif /* value_h__ */

